<?php


class ABT_Options  {

    public $citation_styles;

    /**
     * Save the array of citation styles to $this->citation_styles and add a
     *   link to the options page in the admin menu.
     *
     * @since 3.0.0
     */
    public function __construct() {
        $this->get_citation_styles();
        add_action('admin_menu', array($this, 'add_options_page'));
    }

    /**
     * Grabs the citation styles from the vendor array and saves them to
     *   $this->citation_styles
     *
     * @since 3.0.0
     * @return void
     */
    private function get_citation_styles() {
        include dirname(__FILE__).'/../vendor/citationstyles.php';
        $this->citation_styles = $citation_styles;
    }

    /**
     * Instantiates the options page.
     *
     * @since 3.0.0
     */
    public function add_options_page() {
        add_options_page(
            "Academic Blogger's Toolkit Options",
            "Academic Blogger's Toolkit",
            'manage_options',
            'abt-options',
            array($this, 'ABT_options_page')
        );
    }

    /**
     * Renders the options page.
     *
     * @since 3.0.0
     */
    public function ABT_options_page() {

        if (!current_user_can('manage_options')) {
            wp_die('You do not have sufficient permissions to access this page.');
        }

        $abt_options = get_option('abt_options');

        $hidden_field_1 = isset($_POST['abt_custom_css_editor_form_submitted']) ? esc_html($_POST['abt_custom_css_editor_form_submitted']) : '';
        $hidden_field_2 = isset($_POST['abt_citation_style_form_submitted']) ? esc_html($_POST['abt_citation_style_form_submitted']) : '';
        $hidden_field_3 = isset($_POST['abt_display_options_form_submitted']) ? esc_html($_POST['abt_display_options_form_submitted']) : '';

        if ($hidden_field_1 == 'Y') {
            $abt_options['custom_css'] = stripslashes_deep($_POST['abt_custom_css_editor']);
            update_option('abt_options', $abt_options);
        }

        if ($hidden_field_2 == 'Y') {
            $abt_options['abt_citation_style'] = esc_html($_POST['abt_citation_style']);
            update_option('abt_options', $abt_options);
        }

        if ($hidden_field_3 == 'Y') {
            $abt_options['display_options']['bibliography'] = esc_html($_POST['abt_bibliography_display']);
            $abt_options['display_options']['PR_boxes'] = esc_html($_POST['abt_PR_display']);
            $abt_options['display_options']['links'] = esc_html($_POST['abt_link_display']);
            $abt_options['display_options']['bib_heading'] = esc_html($_POST['abt_bib_heading']);
            update_option('abt_options', $abt_options);
        }

        $abt_saved_css = isset($abt_options['custom_css']) ? $abt_options['custom_css'] : '';
        $selected_style = isset($abt_options['abt_citation_style']) ? $abt_options['abt_citation_style'] : 'american-medical-association';

        $selected_bib_display = isset($abt_options['display_options']['bibliography']) ? $abt_options['display_options']['bibliography'] : '';
        $selected_PR_box_display = isset($abt_options['display_options']['PR_boxes']) ? $abt_options['display_options']['PR_boxes'] : '';
        $selected_link_display = isset($abt_options['display_options']['links']) ? $abt_options['display_options']['links'] : '';
        $bib_heading = isset($abt_options['display_options']['bib_heading']) ? $abt_options['display_options']['bib_heading'] : '';

        ?>
		    <div class="wrap"><h2><?php _e("Academic Blogger's Toolkit Options", 'academic-bloggers-toolkit') ?></h2><div id="poststuff"><div id="post-body" class="metabox-holder"><div id="post-body-content"><div class="meta-box-sortables ui-sortable"><div class="postbox"><h3><?php _e('Please send your feedback!', 'academic-bloggers-toolkit') ?></h3><div class="inside"><?php _e('If you experience a bug or would like to request a new feature, please visit the ', 'academic-bloggers-toolkit') ?><a href='https://github.com/dsifford/academic-bloggers-toolkit' target='_blank'> <?php _e('GitHub Repository', 'academic-bloggers-toolkit') ?></a><?php _e('and submit an issue. I\'ll do my best to get it handled in a timely manner. Comments can also be sent to me on twitter ', 'academic-bloggers-toolkit') ?><a href='http://www.twitter.com/flightmed1' target='_blank'>@flightmed1</a>.</div></div><div class="postbox"><h3><?php _e('Preferred Citation Style', 'academic-bloggers-toolkit') ?></h3><div class="inside"><form method="post" name="abt_citation_style_form"><input type="hidden" name="abt_citation_style_form_submitted" value="Y"/><select id="abt_citation_style" name="abt_citation_style"><?php if ($this->citation_styles) : foreach ($this->citation_styles as $key => $value) : $■['key'] = $key;$■['value'] = $value; ?><option value="<?php echo $value['value'] ?>" <?php selected($selected_style, $value['value']); ?>><?php echo $value['label'] ?></option><?php endforeach; endif ?></select><input type="submit" name="abt_citation_style_submit" value="Update" class="button-primary"/></form></div></div><div class="postbox"><h3><?php _e('Display Options', 'academic-bloggers-toolkit') ?></h3><div class="inside"><form method="post" name="abt_display_options_form" value="Y"><input type="hidden" name="abt_display_options_form_submitted" value="Y"/><table style="width: 100%;"><tr><td style="width: 25%;"><label for="abt_bibliography_display"><?php _e('The bibliography should', 'academic-bloggers-toolkit') ?></label></td><td style="width: 75%;"><select name="abt_bibliography_display" style="width: 100%;"><option value="fixed" <?php selected($selected_bib_display, 'fixed'); ?>><?php _e('be fixed on the page', 'academic-bloggers-toolkit') ?></option>
<option value="toggle" <?php selected($selected_bib_display, 'toggle'); ?>><?php _e('toggle on click', 'academic-bloggers-toolkit') ?></option></select></td></tr><tr><td style="width: 25%;"><label for="abt_PR_display"><?php _e('The peer review boxes should', 'academic-bloggers-toolkit') ?></label></td><td style="width: 75%;"><select name="abt_PR_display" style="width: 100%;"><option value="toggle" <?php selected($selected_PR_box_display, 'toggle'); ?>><?php _e('toggle on click', 'academic-bloggers-toolkit') ?></option>
<option value="fixed" <?php selected($selected_PR_box_display, 'fixed'); ?>><?php _e('be fixed on the page', 'academic-bloggers-toolkit') ?></option></select></td></tr><tr><td style="width: 25%;"><label for="abt_link_display"><?php _e('Clickable links should be included in the bibliography', 'academic-bloggers-toolkit') ?></label></td><td style="width: 75%;"><select name="abt_link_display" style="width: 100%;"><option value="always" <?php selected($selected_link_display, 'always'); ?>><?php _e('whenever possible', 'academic-bloggers-toolkit') ?></option>
<option value="urls" <?php selected($selected_link_display, 'urls'); ?>><?php _e('when the citation has a visible URL', 'academic-bloggers-toolkit') ?></option>
<option value="never" <?php selected($selected_link_display, 'never'); ?>><?php _e('never', 'academic-bloggers-toolkit') ?></option></select></td></tr><tr><td style="width: 25%;"><label style="white-space: nowrap;" for="abt_bib_heading"><?php _e('The heading for bibliographies should be', 'academic-bloggers-toolkit') ?></label></td><td style="width: 75%;"><input type="text" name="abt_bib_heading" value="<?php echo $bib_heading ?>" class="large-text"/></td></tr><tr><td colspan="2" style="text-align: right;"><input type="submit" name="abt_display_options_submit" value="Update" class="button-primary"/></td></tr></table></form></div></div><div class="postbox"><h3><?php _e('Override CSS', 'academic-bloggers-toolkit') ?></h3><table class="form-table"><tr valign="top"><td scope="row" width="50%"><form method="post" name="abt_custom_css_editor_form"><input type="hidden" name="abt_custom_css_editor_form_submitted" value="Y"/><textarea name="abt_custom_css_editor" cols="80" rows="20" style="font-family: monospace;" class="large-text"><?php echo $abt_saved_css ?></textarea><br/><input type="submit" name="abt_custom_css_submit" value="Update" class="button-primary"/></form></td><td width="50%" style="vertical-align: top;"><table><tr><th colspan="2"><?php _e('CSS Selectors used in this plugin', 'academic-bloggers-toolkit') ?></th></tr><tr><td><strong><?php _e('Inline Citations:', 'academic-bloggers-toolkit') ?></strong></td><td><code>.abt-cite</code></td></tr><tr><td><strong><?php _e('Peer Review Boxes:', 'academic-bloggers-toolkit') ?></strong></td><td><code>abt-PR-heading, .abt-PR-info, .abt-PR-headshot, .abt-chat-bubble</code></td></tr><tr><td><strong><?php _e('Citation Tooltips:', 'academic-bloggers-toolkit') ?></strong></td><td><code>.abt-tooltip, .abt-tooltip-arrow, .abt-tooltip-touch-close</code></td></tr><tr><td><strong><?php _e('Bibliography:', 'academic-bloggers-toolkit') ?></strong></td><td><code>#abt-smart-bib, #abt-smart-bib h3, #abt-smart-bib > div, .abt-url</code></td></tr><tr><td><strong><?php _e('Footnotes', 'academic-bloggers-toolkit') ?></strong></td><td><code>#abt-footnote, .abt-footnote-heading, .abt-footnote-item, .abt-footnote-number</code></td></tr></table></td></tr></table></div><div class="postbox"><h3><?php _e('How do I', 'academic-bloggers-toolkit') ?>...</h3><div class="inside"><div style="display: flex; flex-wrap: wrap;"><div style="flex: 1; min-width: 300px; padding: 0 5px;"><strong><?php _e('Make my tooltips a different color?', 'academic-bloggers-toolkit') ?></strong><p data-height="300" data-theme-id="dark" data-slug-hash="pboYoZ" data-default-tab="result" data-user="dsifford" data-embed-version="2" class="codepen">See the Pen <a href="http://codepen.io/dsifford/pen/pboYoZ/">pboYoZ</a> by Derek Sifford (<a href="http://codepen.io/dsifford">@dsifford</a>) on <a href="http://codepen.io">CodePen</a>.</p><script async src="//assets.codepen.io/assets/embed/ei.js"></script></div><div style="flex: 1; min-width: 300px; padding: 0 5px;"><strong><?php _e('Apply style to the bibliography list?', 'academic-bloggers-toolkit') ?></strong><p data-height="300" data-theme-id="dark" data-slug-hash="JKjzGj" data-default-tab="result" data-user="dsifford" data-embed-version="2" class="codepen">See the Pen <a href="http://codepen.io/dsifford/pen/JKjzGj/">Bibliography</a> by Derek Sifford (<a href="http://codepen.io/dsifford">@dsifford</a>) on <a href="http://codepen.io">CodePen</a>.</p><script async src="//assets.codepen.io/assets/embed/ei.js"></script></div><div style="flex: 1; min-width: 300px; padding: 0 5px;"><strong><?php _e('Change the style of the peer review boxes?', 'academic-bloggers-toolkit') ?></strong><p data-height="300" data-theme-id="dark" data-slug-hash="ezYXzX" data-default-tab="result" data-user="dsifford" data-embed-version="2" class="codepen">See the Pen <a href="http://codepen.io/dsifford/pen/ezYXzX/">Peer Review Boxes</a> by Derek Sifford (<a href="http://codepen.io/dsifford">@dsifford</a>) on <a href="http://codepen.io">CodePen</a>.</p><script async src="//assets.codepen.io/assets/embed/ei.js"></script></div></div></div></div></div></div></div></div></div>
		<?php

    }
}
new ABT_Options();
